@@ -670,11 +670,16 @@ class DcsConnection(ServerConnection):
         xtea = new(self.__cryptoKey__, mode=MODE_CBC,IV=iv)
         payload_decrypted = xtea.decrypt(payload)[8:(len(payload)-2)]
         payload_string = payload_decrypted.decode()
-        return payload_string
+        payload_string_stripped = payload_string.rstrip('\x00')
+        return payload_string_stripped
 
     def _getsetupelements(self, payload_string):
         message_string = payload_string[payload_string.find("?")+1:]
-        setup_string = message_string[0:message_string.find("/")]
+        seperator = message_string.find("/")
+        if seperator != -1:
+            setup_string = message_string[0:seperator]
+        else:
+            setup_string = message_string
         logging.debug("["+str(self._serial )+"@"+str(self._serverAddress)+":"+str(self._serverPort)+"]; Setup changes: " + str(setup_string))
         elements = setup_string.split("&")
         return elements
