@@ -15,12 +15,13 @@ sys.path.append(os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)))
 from mypvdevices.ServerConnection import ServerConnection
 from mypvdevices.Dataprocessor import Dataprocessor
 
-BUILDNR = "1.2112.10"
+BUILDNR = "1.2112.11"
 
 JOINTIMEOUTSECONDS = 5
 MONITORSLEEPTIMESECONDS = 10
 DEFAULTBUSSLEEPTIMESECONDS = 15
 CHANNELDATAMAXIMUMAGESECONDS = 60
+ROUNDINGDIGITS = 2
 
 class GeneralDevice:
     _firmwareversion = "3." + str(BUILDNR)
@@ -572,27 +573,28 @@ class GeneralDevice:
                 # print(str(self._name) + " Integrating " + str(datasetname) + ": dt="+str(round(dt, 4))+" x1="+str(round(x1, 4))+" x0="+str(round(x0, 4))+" oldvalue="+str(round(oldvalue, 4))+" newvalue="+str(round(newvalue, 4))+"  value="+str(round(value, 4)))
             self._logdata[datasetname] = value
 
-    def getlogvalue(self, datasetname, casttype=None):
+    def getlogvalue(self, datasetname, casttype=None, scale=None):
         if self._logdata == None or len(self._logdata) == 0:
             return None
 
+        if casttype not in ["int"] and scale != None:
+            logging.error("Scaling only supported for numbers.")
+            raise Exception("Scaling only supportet for numbers.")
+
         try:
             logdata = self._logdata[datasetname]
             if casttype == "int":
-                return int(logdata)
+                value = int(logdata)
+
+                if scale != None and ( isinstance(scale, int) or isinstance(scale, float)):
+                    value = value * scale
+
+                return value
             else:
                 return logdata
         except Exception:
             logging.warning(str(self._name) + " logvalue not found " + str(datasetname))
             return None
-
-    def getintlogvalue(self, datasetname):      #todo change everywhere to getlogvalue(datasetname, "int") and cleanup
-        # logdata = self.getlogvalue(datasetname)
-        # if logdata != None:
-        #     return int(logdata)
-        # else:
-        #     return None
-        return self.getlogvalue(datasetname, "int")
     
     def clearlog(self):
         self._logdata.clear()
@@ -634,7 +636,7 @@ class GeneralDevice:
             result.append(channelId)
         return result
 
-    def getchannelvalue(self, channel, safemode=False):
+    def getchannelvalue(self, channel, safemode=False, scale=None):
         try:
             if self._registers == None:
                 logging.info("No channeldata available. Registers have not been read yet. Run read registers first. " + str(self._name) + ", " + str(channel))
@@ -690,6 +692,12 @@ class GeneralDevice:
             if config.scale != 0 and ( isinstance(value, int) or isinstance(value, float) ):
                 value = value * config.scale
             
+            if scale != None and scale != 0 and ( isinstance(value, int) or isinstance(value, float) ) and ( isinstance(scale, int) or isinstance(scale, float) ):
+                value = value * scale
+
+            if isinstance(value, float):
+                value = round(value, ROUNDINGDIGITS)
+
             return value
         except Exception as e:
             if safemode:
@@ -766,59 +774,9 @@ class GeneralDevice:
     def getdata(self):
         raise Exception(str(self._name) + ". Not implemented: getdata")
 
-        # t = datetime.now()        #todo remove
-        # data = {
-        #         "fwversion": self._firmwareversion,
-		# 		"power": t.second * 100,
-        #         "temp1": t.minute,
-        #         "check": 9544,
-        #         "temp2": self.getchannelvalue("temp2", True),
-		# 		"loctime": time.strftime("%H:%M:%S")
-        #         }            
-        # return data
-
     def getlogdata(self, time = None):
         raise Exception(str(self._name) + ". Not implemented: getlogdata")
 
-        # logdata = {       #todo remove
-        #     # "i_unknown": 123,
-        #     "time": time,
-        #     "i_power": self.getlogvalue("power", "int"),
-        #     "i_boostpower": 750,
-        #     "i_meterfeed": None,
-        #     "i_metercons": 4,
-        #     "i_temp1": self.getlogvalue("temp1", "int"),
-        #     "i_m0l1": 6,
-        #     "i_m0l2": 7,
-        #     "i_m0l3": 8,
-        #     "i_m1sum": 9,
-        #     "i_m1l1": 10,
-        #     "i_m1l2": 11,
-        #     "i_m1l3": 12,
-        #     "i_m2sum": 13,
-        #     "i_m2l1": 14,
-        #     "i_m2l2": 15,
-        #     "i_m2l3": 16,
-        #     "i_m2soc": 17,
-        #     "i_m3sum": 18,
-        #     "i_m3l1": 19,
-        #     "i_m3l2": 20,
-        #     "i_m3l3": 21,
-        #     "i_m3soc": 22,
-        #     "i_m4sum": 23,
-        #     "i_m4l1": 24,
-        #     "i_m4l2": 25,
-        #     "i_m4l3": 26,
-        #     "s_json" : "27",
-        #     "i_temp2": self.getintlogvalue("temp2"),
-        #     "i_power1": 29,
-        #     "i_power2": 30,
-        #     "i_power3": 31,
-        #     "i_temp3": 32,
-        #     "i_temp4": 33
-        #     }
-        # return logdata
-
 # # Entry Point     
 # if __name__ == "__main__":
 
