@@ -43,14 +43,14 @@ class ModbusConnection:     #Singleton
         self.__connectionCounter__ = dict()
         self.__modbusClient__ = ModbusClient(method = "rtu", port="/dev/ttyUSB0", stopbits = self.__modbus_stopbits, bytesize = self.__modbusbytesize, parity = self.__modbusparity, baudrate= self.__modbusbaudrate, timeout=self.__modbustimeout, retry_on_empty=self.__modbusretryonempty, retry_on_invalid=self.__modbusretryoninvalid)
 
-    def __increaseModbusErrorCounter__(self, nodeId):
+    def __increaseModbusErrorCounter__(self, unitId):
         try:
-            self.__connectionErrorCounter__[nodeId] = self.__connectionErrorCounter__[nodeId] + 1
+            self.__connectionErrorCounter__[unitId] = self.__connectionErrorCounter__[unitId] + 1
         except:
-            self.__connectionErrorCounter__[nodeId] = 1
+            self.__connectionErrorCounter__[unitId] = 1
         
-        logging.debug("[ModbusConnection] Node " + str(nodeId) + ": Communication error. Total errors: " + str(self.__connectionErrorCounter__[nodeId]))
-        counter = self.__connectionErrorCounter__[nodeId]
+        logging.debug("[ModbusConnection] Node " + str(unitId) + ": Communication error. Total errors: " + str(self.__connectionErrorCounter__[unitId]))
+        counter = self.__connectionErrorCounter__[unitId]
         return counter
 
     def setConfiguration(self, stopbits, bytesize, parity, baudrate, timeout, retry_on_empty, retry_on_invalid):
@@ -66,11 +66,11 @@ class ModbusConnection:     #Singleton
             self.__modbusClient__.close()
             self.__modbusClient__ = ModbusClient(method = "rtu", port="/dev/ttyUSB0", stopbits = self.__modbus_stopbits, bytesize = self.__modbusbytesize, parity = self.__modbusparity, baudrate= self.__modbusbaudrate, timeout=self.__modbustimeout, retry_on_empty=self.__modbusretryonempty, retry_on_invalid=self.__modbusretryoninvalid)
 
-    def getModbusErrorCounter(self, nodeId):
+    def getModbusErrorCounter(self, unitId):
         try:
-            return self.__connectionErrorCounter__[nodeId]
+            return self.__connectionErrorCounter__[unitId]
         except:
-            self.__connectionErrorCounter__[nodeId] = 0
+            self.__connectionErrorCounter__[unitId] = 0
             return 0
 
     def getModbusErrorRate(self, node):
@@ -88,11 +88,11 @@ class ModbusConnection:     #Singleton
     def getModbusModuleState(self):
         return self.__modbusModuleState
 
-    def resetCounters(self, nodeId):
+    def resetcounters(self, unitId):
         with self.__mutex__:
             try:
-                self.__connectionErrorCounter__[nodeId] = 0
-                self.__connectionCounter__[nodeId] = 0
+                self.__connectionErrorCounter__[unitId] = 0
+                self.__connectionCounter__[unitId] = 0
             except Exception as e:
                 logging.error("[ModbusConnection] Node " + str(node) + ": Error resetting counters. " + str(e))
 
@@ -125,7 +125,9 @@ class ModbusConnection:     #Singleton
                     socket = self.__connectSocket__()
                 except Exception as e:
                     logging.warning("[ModbusConnection] Modbus socket error: " + str(e))
-                    socket = None
+                    self.__increaseModbusErrorCounter__(node)
+                    raise e
+                #     socket = None
                 if(socket):
                     try:
                         result = self.__modbusClient__.write_register(registerId, value, unit = node)
@@ -139,10 +141,39 @@ class ModbusConnection:     #Singleton
                     logging.warning("[ModbusConnection] Cannot write to node " + str(node) + ".")
         return result
 
-    def readregisters(self, node, startAddress = 1000, registersToRead = 1):
-        return self.__readregisters__(node, startAddress, registersToRead, "holding")
+    def writeregisters(self, node, registerStartId, values):
+        result = None
+        with self.__mutex__:
+            logging.debug("[ModbusConnection] Node " + str(node) + ": writing registers starting at" + str(registerStartId) +" with value "+ str(values))
+            try:
+                self.__connectionCounter__[node] += 1
+            except:
+                self.__connectionCounter__[node] = 1
+            if self.__modbusClient__ != None:
+                try:
+                    socket = self.__connectSocket__()
+                except Exception as e:
+                    logging.warning("[ModbusConnection] Modbus socket error: " + str(e))
+                    self.__increaseModbusErrorCounter__(node)
+                    raise e
+                #     socket = None
+                if(socket):
+                    try:
+                        result = self.__modbusClient__.write_registers(registerStartId, values, unit = node)
+                        if result.isError():
+                            raise Exception(str(result))
+                    except Exception as e:
+                        logging.warning("[ModbusConnection] Modbus Write Error: " + str(e))
+                        self.__increaseModbusErrorCounter__(node)
+                        result = None
+                else:
+                    logging.warning("[ModbusConnection] Cannot write to node " + str(node) + ".")
+        return result
 
-    def __readregisters__(self, node, startAddress, registersToRead, type="holding"):
+    def readregisters(self, node, startAddress = 1000, registersToRead = 1):
+        return self._readregisters(node, startAddress, registersToRead, "holding")
+
+    def _readregisters(self, node, startAddress, registersToRead, type="holding"):
         with self.__mutex__:
             logging.debug("[ModbusConnection] Node " + str(node) + ": Reading " + str(registersToRead) + " " + str(type) + "-registers starting with register "+ str(startAddress))
             try:
@@ -183,7 +214,7 @@ class ModbusConnection:     #Singleton
             return None
 
     def readinputregisters(self, node, startAddress = 1000, registersToRead = 1):
-        return self.__readregisters__(node, startAddress, registersToRead, "input")
+        return self._readregisters(node, startAddress, registersToRead, "input")
 
 
 # Entry Point     
@@ -234,7 +265,7 @@ if __name__ == "__main__":
             print(color('ERROR: Read temperature value in valid range. No data found. ' + str(e), fore='red', style='bright'))
 
     # reading multiple registers
-    ModbusConnection.instance().resetCounters(unitToTest)
+    ModbusConnection.instance().resetcounters(unitToTest)
     register = ModbusConnection.instance().readregisters(unitToTest, 1000, 24)
     errors = ModbusConnection.instance().getModbusErrorCounter(unitToTest)
     errorrate =  ModbusConnection.instance().getModbusErrorRate(unitToTest)
@@ -244,7 +275,7 @@ if __name__ == "__main__":
         print(color('ERROR: Valid multiple register read.', fore='red', style='bright'))
 
     # reading multiple registers again
-    ModbusConnection.instance().resetCounters(unitToTest)
+    ModbusConnection.instance().resetcounters(unitToTest)
     register = ModbusConnection.instance().readregisters(unitToTest, 1000, 24)
     errors = ModbusConnection.instance().getModbusErrorCounter(unitToTest)
     errorrate =  ModbusConnection.instance().getModbusErrorRate(unitToTest)
@@ -270,7 +301,7 @@ if __name__ == "__main__":
         print(color('SUCCESS: Node 0 (Broadcast) read.', fore='green', style='bright'))
 
     # reading of multiple invalid registers
-    ModbusConnection.instance().resetCounters(unitToTest)
+    ModbusConnection.instance().resetcounters(unitToTest)
     register = ModbusConnection.instance().readregisters(unitToTest, 4130, 24)
     errors = ModbusConnection.instance().getModbusErrorCounter(unitToTest)
     errorrate =  ModbusConnection.instance().getModbusErrorRate(unitToTest)
@@ -280,7 +311,7 @@ if __name__ == "__main__":
         print(color('ERROR: Multiple invalid register read.', fore='red', style='bright'))
 
     # reading multiple registers multiple times
-    ModbusConnection.instance().resetCounters(unitToTest)
+    ModbusConnection.instance().resetcounters(unitToTest)
     temp = 0
     for i in range(0, 50):
         register = ModbusConnection.instance().readregisters(unitToTest, 1000, 24)
@@ -306,7 +337,7 @@ if __name__ == "__main__":
         print(color('ERROR: Configuration change test.', fore='red', style='bright'))
 
     # writing to vaild node
-    ModbusConnection.instance().resetCounters(unitToTest)
+    ModbusConnection.instance().resetcounters(unitToTest)
     register = 1023
     value = 1514
     try:
@@ -324,7 +355,7 @@ if __name__ == "__main__":
     node = 99
     register = 1023
     value = 1514
-    ModbusConnection.instance().resetCounters(node)
+    ModbusConnection.instance().resetcounters(node)
     try:
         result = ModbusConnection.instance().writeregister(node, register, value)
         errors = ModbusConnection.instance().getModbusErrorCounter(node)
